PetBattleTeams = CreateFrame("frame")
local START = 1
local FINISHED = 5
local PETS_PER_TEAM = 3



--tooltip functions

--Creates fake of blizzards PetBattleUnitTooltip
--Returns a tooltip
PetBattleTeams.CreateTooltip = function(self)
	local tooltip = CreateFrame("frame","PetBattleTeamTooltip",nil,"PetBattleUnitTooltipTemplate")
	tooltip:SetHeight(211)
	tooltip.WeakToLabel:Hide()
	tooltip.ResistantToLabel:Hide()
	
	--icon quality glow
	tooltip.rarityGlow = tooltip:CreateTexture("PetBattleTeamTooltipGlow","OVERLAY")
	tooltip.rarityGlow:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
	tooltip.rarityGlow:SetBlendMode("ADD")
	tooltip.rarityGlow:ClearAllPoints()
	tooltip.rarityGlow:SetDrawLayer("OVERLAY", 0)
	tooltip.rarityGlow:SetWidth(tooltip.Icon:GetWidth() * 1.7)
	tooltip.rarityGlow:SetHeight(tooltip.Icon:GetHeight() * 1.7)
	tooltip.rarityGlow:SetPoint("CENTER", tooltip.Icon, "CENTER", 0, 0)
	
	--team indicator
	tooltip.teamText = tooltip:CreateFontString("PetBattleTeamTooltipTeamText","OVERLAY","GameFontNormalSmall")
	tooltip.teamText:SetJustifyH("CENTER")
	tooltip.teamText:SetText("Team XX")
	tooltip.teamText:SetTextColor(1,1,1)
	tooltip.teamText:SetSize(0,0)
	tooltip.teamText:SetFont("Fonts\\FRIZQT__.TTF",12,"OUTLINE")
	tooltip.teamText:SetPoint("BOTTOMLEFT",tooltip.Name,"TOPLEFT",0,2)

	
	tooltip.AbilitiesLabel:Show()
	tooltip.XPBar:Show()
	tooltip.XPBG:Show()
	tooltip.XPBorder:Show()
	tooltip.XPText:Show()
	tooltip.teamText:Show()
	
	return tooltip
end

--Sets the tooltip to the currently moused over unit
PetBattleTeams.ToolTipSetUnit = function(self,tooltip, petID, team, petIndex)
	
	local speciesID, customName, level, xp, maxXp, displayID, petName, petIcon, petType, creatureID = C_PetJournal.GetPetInfoByPetID(petID)
	if not speciesID then return end
	
	local pet = PetBattleTeamsSettings.teams[team][petIndex]
	local health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petID)
	local r, g, b,hex = GetItemQualityColor(rarity-1)
	
	tooltip.rarityGlow:SetVertexColor(r, g, b)
	tooltip.Icon:SetTexture(petIcon)
	tooltip.Name:SetText("|c"..hex..petName.."|r")
	tooltip.Level:SetText(level)
	tooltip.XPBar:SetWidth(max((xp / max(maxXp,1)) * tooltip.xpBarWidth, 1))
	tooltip.Delimiter:SetPoint("TOP", tooltip.XPBG, "BOTTOM", 0, -10)
	tooltip.XPText:SetFormattedText(tooltip.xpTextFormat or PET_BATTLE_CURRENT_XP_FORMAT, xp, maxXp)
	tooltip.teamText:SetText("Team "..team)
	tooltip.AttackAmount:SetText(attack)
	tooltip.SpeedAmount:SetText(speed)
	tooltip.PetType.Icon:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[petType])
	
	tooltip.SpeciesName:Hide()
	if customName then
		if ( customName ~= petName ) then
			tooltip.Name:SetText(customName)
			tooltip.SpeciesName:SetText(petName)
			tooltip.SpeciesName:Show()
		end
	end
	
	if ( tooltip.HealthText ) then
		tooltip.HealthText:SetFormattedText(tooltip.healthTextFormat or PET_BATTLE_CURRENT_HEALTH_FORMAT, health, maxHealth)
	end
	
	if ( health == 0 ) then
		tooltip.ActualHealthBar:SetWidth(1)
	else
		tooltip.ActualHealthBar:SetWidth((health / max(maxHealth,1)) * tooltip.healthBarWidth)
	end
		
	for i=1, #pet.abilities do
		local name, icon, petType = C_PetJournal.GetPetAbilityInfo(pet.abilities[i])
		local abilityIcon = tooltip["AbilityIcon"..i]
		local abilityName = tooltip["AbilityName"..i]

		abilityName:SetShown(name)
		abilityIcon:SetShown(icon)
		abilityIcon:SetTexture("Interface\\PetBattles\\BattleBar-AbilityBadge-Neutral")
		if name then 
			abilityName:SetText(name)
		end
	end
end

--right click config menu initialization
PetBattleTeams.DropDownOnLoad = function(self,level)
	local info = UIDropDownMenu_CreateInfo()
	info.isNotRadio = true
	
	info.text = "Attach to Pet Journal"
	info.func = function(_, _, _, value) 
		PetBattleTeamsSettings.attached = not value
		
		if PetBattleTeamsSettings.attached then
			PetBattleTeamsSettings.LockLocation = false;
		end
		
		PetBattleTeams:Attach(PetBattleTeamsSettings.attached)
		PetBattleTeams:LockLocation(PetBattleTeamsSettings.LockLocation)
		
	end
	info.checked = function() return PetBattleTeamsSettings.attached end
	UIDropDownMenu_AddButton(info, level)
	
	info.text = "Lock Location"
	info.func = function(_, _, _, value)  
		PetBattleTeamsSettings.LockLocation = not value 
		PetBattleTeams:LockLocation(PetBattleTeamsSettings.LockLocation)
	end
	info.checked = function()  return PetBattleTeamsSettings.LockLocation end
	UIDropDownMenu_AddButton(info, level)
	
	info.text = "Display Level"
	info.func = function(_, _, _,value) 
		PetBattleTeamsSettings.displayLevel = not value 
		PetBattleTeams:UpdateAllTeams()  
	end
	info.checked = function() return PetBattleTeamsSettings.displayLevel   end
	UIDropDownMenu_AddButton(info, level)
end

--create parent frame
PetBattleTeams.CreateParentFrame = function(self,parentName)
	if not parentName then error("parentName is nil") end
	
	local frame = CreateFrame("FRAME", parentName, UIParent)
	
	--considering moving this out of the function
	--frame:SetPoint("TOPLEFT",UIParent,"TOPRIGHT",-5,4)
	
	frame:SetClampedToScreen(true)
	frame:SetFrameStrata("HIGH")
	frame:SetSize(165,597)
	frame:RegisterForDrag("LeftButton")
	frame:EnableMouse(true)
	frame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = false, tileSize = 32, edgeSize=32, insets ={left=8,right=8,top=8,bottom=8}})
	frame:SetBackdropColor(.1,.1,.1,1)
	
	--right click menu
	frame.menu = CreateFrame("Frame",parentName,frame,"UIDropDownMenuTemplate")
	frame.menu:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT",0,0)
	UIDropDownMenu_Initialize(frame.menu, PetBattleTeams.DropDownOnLoad, "MENU")
	
	--frame label
	frame.name = frame:CreateFontString(parentName.."Name","OVERLAY","GameFontNormal")
	frame.name:SetJustifyH("CENTER")
	frame.name:SetText("Pet Battle Teams")
	frame.name:SetSize(0,0)
	frame.name:SetPoint("CENTER",frame,"TOP",0,-15)
	
	frame:SetScript("OnMouseUp", function(self,button) 
		if button == "RightButton" then
			local scale  = UIParent:GetEffectiveScale()
			local cursorX, cursorY = GetCursorPosition()
			ToggleDropDownMenu(1, nil, self.menu, nil, cursorX/scale, cursorY/scale + 15/scale )
		end
	end)
	
	return frame
end

PetBattleTeams.UpdateBgFrame = function(self)
	if self.bgFrame:GetParent() == PetJournalParent then
		self.bgFrame:SetSize(165,614)
	else
		self.bgFrame:SetSize(165,46*(#PetBattleTeamsSettings.teams)+35)
		if #PetBattleTeamsSettings.teams < self.maxTeams then
			self.bgFrame:SetSize(165,46*(#PetBattleTeamsSettings.teams+1)+35)
		end
	end
end


--/run PetBattleTeams:Attach(true)
PetBattleTeams.Attach = function(self,attach)
	if attach then
		print("attached!")
		PetBattleTeamsSettings.LockLocation = false;
		self.bgFrame:SetParent("PetJournalParent")
		self.bgFrame:ClearAllPoints();
		self.bgFrame:SetPoint("TOPLEFT",PetJournalParent,"TOPRIGHT",-5,4)
	else
		self.bgFrame:SetParent("UIParent")
	end
	self:UpdateBgFrame()
end

PetBattleTeams.LockLocation = function(self,lock)
	if lock then 
		self.bgFrame:SetMovable(false)
		self.bgFrame:SetScript("OnDragStart", nil)
		self.bgFrame:SetScript("OnDragStop", nil)
	else
		self.bgFrame:SetScript("OnDragStart", self.bgFrame.StartMoving)
		self.bgFrame:SetScript("OnDragStop", self.bgFrame.StopMovingOrSizing)
		self.bgFrame:SetMovable(true)
	end
end


--Add team button related functions
PetBattleTeams.CreateAddTeamButton = function(self,parentName)
	local button = CreateFrame("BUTTON", parentName, self.bgFrame)
	button:SetSize(40,40)
	
	button.icon = button:CreateTexture(parentName.."Icon","ARTWORK")
	button.icon:SetTexture("Interface\\GuildBankFrame\\UI-GuildBankFrame-NewTab")
	button.icon:SetAllPoints(button)
	
	button:SetScript("OnClick",function(self) 
		PetBattleTeams:CreateNewTeam()
		PetBattleTeams:UpdateAllTeams()
	end)
	return button
end


PetBattleTeams.UpdateAddTeamButton = function(self)
	if self.addTeamButton then
		self.addTeamButton:Hide()
		if #PetBattleTeamsSettings.teams == 0 then
			self.addTeamButton:SetPoint("TOPLEFT", self.bgFrame ,"TOPLEFT",17,-30)
			self.addTeamButton:Show()
		elseif	#PetBattleTeamsSettings.teams < self.maxTeams and #PetBattleTeamsSettings.teams > 0 then
			self.addTeamButton:SetPoint("TOPLEFT","PetTeam"..#PetBattleTeamsSettings.teams.."Button"..1,"BOTTOMLEFT",-2,0)
			self.addTeamButton:Show()
		end
	end
end

--pet unitframe functions
PetBattleTeams.CreatePetButton = function(self,team,petIndex)
	
	local parentName = "PetTeam"..team.."Button"..petIndex
	local button = CreateFrame("BUTTON", parentName, self.bgFrame, "PetBattleMiniUnitFrameAlly")

	--fix template sizes
	button:SetSize(40,40)
	button.healthBarWidth = 37
	button.BorderAlive:SetSize(50,50)
	button.BorderDead:SetSize(50,50)
	
	if not button.rarityGlow then button.rarityGlow = button:CreateTexture(parentName.."Glow","ARTWORK") end

	button.rarityGlow:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
	button.rarityGlow:SetBlendMode("ADD")
	button.rarityGlow:ClearAllPoints()
	button.rarityGlow:SetDrawLayer("ARTWORK", 0)
	button.rarityGlow:SetWidth(button:GetWidth() * 1.6)
	button.rarityGlow:SetHeight(button:GetHeight() * 1.4)
	button.rarityGlow:SetPoint("CENTER", button.Icon, "CENTER", 0, 4)
	
	--[[button.levelBubble = button:CreateTexture(parentName.."LevelBubble","OVERLAY")
	button.levelBubble:SetTexture("Interface\\PetBattles\\PetJournal")
	button.levelBubble:SetSize(21,21)
	button.levelBubble:SetTexCoord(0.06835938,0.10937500,0.02246094,0.04296875)
	button.levelBubble:SetPoint("CENTER",button,"TOPLEFT",8,-8)
	--]]
	button.level = button:CreateFontString(parentName.."Level","OVERLAY","GameFontNormalSmall")
	button.level:SetJustifyH("CENTER")
	button.level:SetText("XX")
	button.level:SetSize(0,0)
	button.level:SetFont("Fonts\\FRIZQT__.TTF",11,"OUTLINE")
	button.level:SetPoint("BOTTOMLEFT",button.Icon,"BOTTOMLEFT",2,10)
	
	
	
	button.selected = button:CreateTexture(parentName.."Selected","OVERLAY") 
	button.selected:SetSize(36,36)
	button.selected:SetTexture("Interface\\PetBattles\\PetJournal")
	button.selected:SetTexCoord(0.11328125,0.16210938,0.02246094,0.04687500)
	button.selected:SetPoint("CENTER",button,"RIGHT",0,0)
	button.selected:Hide()
	
	button:SetScript("OnLoad",nil)
	button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	button:SetScript("OnClick", function(self,button) 
		local info, petID = GetCursorInfo();
		
		
		if ( button == "RightButton" ) then
				StaticPopup_Show("PBT_TEAM_DELETE",self.team,nil,self.team)
		elseif button == "LeftButton" then
			if info and info == "battlepet" then
				if PetBattleTeams.CursorStruct then
					
					PetBattleTeams:UpdateTeamSwapPets(self.team,self.petIndex, PetBattleTeams.CursorStruct.sourceTeam,PetBattleTeams.CursorStruct.soucePetIndex);
				else
					PetBattleTeams:UpdateTeamNewPet(petID,self.team,self.petIndex) -- UpdateTeamNewPet =function(self,petID,team,petIndex)
				end
				ClearCursor()
				PetBattleTeams.CursorStruct = nil;
			else
				PetBattleTeams:SetTeam(self.team)
			end
		end
	end)
	
	button:SetScript("OnReceiveDrag", function(self,button) 
		local info, petID = GetCursorInfo();
		if info and info == "battlepet" then
			if PetBattleTeams.CursorStruct then
				
				PetBattleTeams:UpdateTeamSwapPets(self.team,self.petIndex, PetBattleTeams.CursorStruct.sourceTeam,PetBattleTeams.CursorStruct.soucePetIndex);
			else
				PetBattleTeams:UpdateTeamNewPet(petID,self.team,self.petIndex) -- UpdateTeamNewPet =function(self,petID,team,petIndex)
			end
			ClearCursor()
			PetBattleTeams.CursorStruct = nil;
		end
	end)
	
	button:RegisterForDrag("LeftButton")
	button:SetScript("OnDragStart", function(self,button)
		PetBattleTeams.CursorStruct = {}
		PetBattleTeams.CursorStruct.sourceTeam = self.team;
		PetBattleTeams.CursorStruct.soucePetIndex = self.petIndex;
		C_PetJournal.PickupPet(self.petID, false);
	end)

	button:SetScript("OnEnter",function(self) 
		PetBattleUnitTooltip_Attach(PetBattleTeams.tooltip, "TOPLEFT", self, "BOTTOMRIGHT", 0, 0)
		local left, bottom, width, height = PetBattleTeams.tooltip:GetBoundsRect()
		
		if left + width > GetScreenWidth() then 
			PetBattleUnitTooltip_Attach(PetBattleTeams.tooltip, "TOPRIGHT", self, "BOTTOMLEFT", 0, 0)
		end
		if bottom < 0 then
			PetBattleUnitTooltip_Attach(PetBattleTeams.tooltip, "BOTTOMRIGHT", self, "TOPLEFT", 0, 0)
		end
		
		if C_PetJournal.GetPetInfoByPetID(button.petID) then
			PetBattleTeams:ToolTipSetUnit(PetBattleTeams.tooltip, self.petID, self.team, self.petIndex)
			PetBattleTeams.tooltip:Show()
		end
	end)
	button:SetScript("OnLeave",function(self)
		if ( PetBattleTeams.tooltip:GetParent() == self ) then
			PetBattleTeams.tooltip:Hide()
		end
	end)
	button:UnregisterAllEvents()
	return button
end

PetBattleTeams.CreatePetUnitFrames = function(self)
	local unitFrames = {}
	for team=1,self.maxTeams do 
		local buttons = {}
		for petIndex =1, PETS_PER_TEAM do 
			button = PetBattleTeams:CreatePetButton(team,petIndex)
			
			if petIndex==1 and team == 1 then
				
				button:SetPoint("TOPLEFT", self.bgFrame ,"TOPLEFT",17,-30)
			elseif petIndex == 1 and team > 1 then
				button:SetPoint("TOPLEFT","PetTeam"..(team-1).."Button"..petIndex,"BOTTOMLEFT",0,-4)
			else
				button:SetPoint("LEFT","PetTeam"..team.."Button"..petIndex-1,"RIGHT",5,0)
			end
			
			table.insert(buttons,button)
		end
		table.insert(unitFrames,buttons)
	end
	
	return unitFrames
end

PetBattleTeams.UpdateAllTeams = function(self)
	for i=1, self.maxTeams do
		self:UpdatePetTeam(i)
	end
	self:UpdateAddTeamButton()
	self:UpdateBgFrame()
end

PetBattleTeams.UpdatePetTeam = function(self,team)
	for i=1,PETS_PER_TEAM do
		self:UpdatePetButton(team,i)
	end
end

PetBattleTeams.UpdatePetButton=function(self,team,petIndex)
	if team < 1 or team > self.maxTeams or petIndex < 1 or petIndex > PETS_PER_TEAM then return end
	
	button = self.petUnitFrames[team][petIndex]
	
	button.team = team
	button.petIndex = petIndex
	button:Hide()
	
	if PetBattleTeamsSettings.teams[button.team] and PetBattleTeamsSettings.teams[button.team][button.petIndex] then
		button:Show()
		
		button.selected:Hide()
		button.Icon:SetTexture("\Interface\\ICONS\\INV_Misc_QuestionMark")
		button.ActualHealthBar:Hide()
		button.rarityGlow:Hide()
		button.BorderAlive:Show()
		button.level:Hide()
		button.BorderDead:Hide()
		button.petID = PetBattleTeamsSettings.teams[button.team][button.petIndex].petID
		if button.petIndex == PETS_PER_TEAM and PetBattleTeamsSettings.selected == button.team then
			
				button.selected:Show()
		end
		
		if button.petID and button.petID > 0 and C_PetJournal.GetPetInfoByPetID(button.petID) then

			-- get info about the pet
			local speciesID, customName, level, xp, maxXp, displayID, name, icon, petType, creatureID, sourceText, description, isWild, canBattle, tradable = C_PetJournal.GetPetInfoByPetID(button.petID)
			local health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(button.petID)
			
			if rarity then
				local r, g, b = GetItemQualityColor(rarity-1)
				button.rarityGlow:SetVertexColor(r, g, b)
			end
			
			button.level:Hide()
			if  PetBattleTeamsSettings.displayLevel then
				button.level:SetText(level)
				button.level:Show()
			end
			
			button.Icon:SetTexture(icon)

			--set health bar
			if health and maxHealth then
				if  ( health == 0 ) then
					button.ActualHealthBar:Hide()
				else
					button.ActualHealthBar:Show()
				end
				button.ActualHealthBar:SetWidth((health / max(maxHealth,1)) * button.healthBarWidth)
			end
			
			--Set alive border
			if health and ( health == 0 ) then
				button.BorderAlive:Hide()
				button.rarityGlow:Hide()
			else
				button.BorderAlive:Show()
				button.rarityGlow:Show()
			end
			
			--set dead border
			if health and ( health == 0 ) then
				button.BorderDead:Show()
				button.rarityGlow:Hide()
			else
				button.BorderDead:Hide()
				button.rarityGlow:Show()
			end
			
			for _, object in pairs(button.hideWhenDeadList) do
				if ( health == 0 ) then
					object:Hide()
				else
					object:Show()
				end
			end
		end
	end
	
end


--high level team functions
PetBattleTeams.RemoveTeam = function(self,team)
	
	table.remove(PetBattleTeamsSettings.teams,team)
	
	if #PetBattleTeamsSettings.teams > 0 then
		PetBattleTeams:SetTeam(1)
	end

	if #PetBattleTeamsSettings.teams < 14 then
			self.addTeamButton:Show()
	end
end



PetBattleTeams.CreateNewTeam = function(self) 
	if #PetBattleTeamsSettings.teams < self.maxTeams then
		local team = {}
		for i = 1,PETS_PER_TEAM do
			pet = {}
			pet.abilities = {}
			pet.petID, pet.abilities[1], pet.abilities[2], pet.abilities[3] = C_PetJournal.GetPetLoadOutInfo(i)
			
				table.insert(team,pet)
			
		end
		if #team > 0 then
			table.insert(PetBattleTeamsSettings.teams,team)
			PetBattleTeamsSettings.selected = #PetBattleTeamsSettings.teams
			if #PetBattleTeamsSettings.teams == 14 then
					self.addTeamButton:Hide()
			end
		end
		
	end
end

PetBattleTeams.UpdateCurrentTeam =function()
	if not PetBattleTeams:IsWorking() then
		local team = {}
		for i = 1,PETS_PER_TEAM do
			pet = {}
			pet.abilities = {}
			pet.petID, pet.abilities[1], pet.abilities[2], pet.abilities[3] = C_PetJournal.GetPetLoadOutInfo(i)
			table.insert(team,pet)
		end
		PetBattleTeamsSettings.teams[PetBattleTeamsSettings.selected] = team
		PetBattleTeams:UpdateAllTeams()
	end
end

PetBattleTeams.UpdateTeamSwapPets=function(self,destinationTeam,destinationPetIndex,sourceTeam,sourcePetIndex)
	
	
	if not PetBattleTeams:IsWorking() then
		petA = PetBattleTeamsSettings.teams[destinationTeam][destinationPetIndex]
		petB = PetBattleTeamsSettings.teams[sourceTeam][sourcePetIndex]
		
		if petA.petID ~= petB.petID then
			for i=1,PETS_PER_TEAM do
				if petA.petID == PetBattleTeamsSettings.teams[sourceTeam][i].petID or petB.petID == PetBattleTeamsSettings.teams[destinationTeam][i].petID then
					return
				end
			end
		end
		PetBattleTeamsSettings.teams[destinationTeam][destinationPetIndex] = petB
		PetBattleTeamsSettings.teams[sourceTeam][sourcePetIndex] = petA
		
		
		
		
		if destinationTeam == PetBattleTeamsSettings.selected or sourceTeam == PetBattleTeamsSettings.selected  then 
			self:SetTeam(PetBattleTeamsSettings.selected)
		end
		PetBattleTeams:UpdateAllTeams()
	end
end

PetBattleTeams.UpdateTeamNewPet =function(self,petID,team,petIndex)
	if not PetBattleTeams:IsWorking() then
		local speciesID, customName, level, xp, maxXp, displayID, name, icon, petType, creatureID, sourceText, description, isWild, canBattle, tradable = C_PetJournal.GetPetInfoByPetID(petID)
		if speciesID and canBattle then
			local pet = {}
			local abilities = {}
			C_PetJournal.GetPetAbilityList(speciesID, abilities, {})
			pet.petID = petID
			pet.abilities = {}
			for i=1,3 do
				pet.abilities[i] = abilities[i]
			end
			PetBattleTeamsSettings.teams[team][petIndex] = pet;
			PetBattleTeams:UpdateAllTeams()
		end
	end
end

PetBattleTeams.SetTeam = function(self,team)
	oldTeam = PetBattleTeamsSettings.selected
	PetBattleTeamsSettings.selected = team
	
	PetBattleTeams:UpdateAllTeams()
	PetBattleTeams:StartWork()
end


--Pet switching logic
PetBattleTeams.OnUpdate = function(self)
	if self.step < 4 then    
		if PetBattleTeamsSettings.teams[PetBattleTeamsSettings.selected] and PetBattleTeamsSettings.teams[PetBattleTeamsSettings.selected][self.step] and C_PetJournal.GetPetInfoByPetID(PetBattleTeamsSettings.teams[PetBattleTeamsSettings.selected][self.step].petID) then
			pet = PetBattleTeamsSettings.teams[PetBattleTeamsSettings.selected][self.step]			
			local petID, ability1, ability2, ability3 = C_PetJournal.GetPetLoadOutInfo(self.step)
			if petID ~= pet.petID then 
				C_PetJournal.SetPetLoadOutInfo(self.step,pet.petID) 
				return
			else
				if ability1 ~= pet.abilities[1] then 
					C_PetJournal.SetAbility(self.step, 1, pet.abilities[1])
					return
				elseif ability2 ~= pet.abilities[2] then
					C_PetJournal.SetAbility(self.step, 2, pet.abilities[2])
					return
				elseif ability3 ~= pet.abilities[3] then
					C_PetJournal.SetAbility(self.step, 3, pet.abilities[3])
					return
				end
			end
			
			if petID == pet.petID and ability1 == pet.abilities[1] and ability2 == pet.abilities[2] and ability3 == pet.abilities[3] then
				self.step = self.step +1
			end
		else --invalid pet / non existant
			C_PetJournal.SetPetLoadOutInfo(self.step,0)
			if C_PetJournal.GetPetLoadOutInfo(self.step) == 0 then
				self.step = self.step +1
			end
		end 
	else
		PetJournal_UpdatePetLoadOut()
		self:SetScript("OnUpdate",nil)
		self.step = FINISHED
	end 
end

PetBattleTeams.StartWork = function(self)
	self.step = START
	self:SetScript("OnUpdate",PetBattleTeams.OnUpdate)
end

PetBattleTeams.IsWorking = function(self)
	return self.step < FINISHED 
end

--Reset saved variables
PetBattleTeams.Reset = function(self,resetTeams)
	print("PetBattleTeamsSettings Reset")
	local backupTeams = PetBattleTeamsSettings.teams
	PetBattleTeamsSettings={}
	if resetTeams then
		PetBattleTeamsSettings.teams = {}
		PetBattleTeams:CreateNewTeam()
	else
		PetBattleTeamsSettings.teams = backupTeams;
	end
	
	PetBattleTeamsSettings.version = PetBattleTeams.version
	PetBattleTeamsSettings.selected = 1
	PetBattleTeamsSettings.displayLevel = false 
	PetBattleTeamsSettings.attached = true
	PetBattleTeamsSettings.LockLocation = true
	
	self:Attach(PetBattleTeamsSettings.attached)
	self:LockLocation(PetBattleTeamsSettings.LockLocation);
	
	
end



PetBattleTeams.CreateUI = function(self)	
	if not self.tooltip then self.tooltip = self:CreateTooltip() end
	if not self.bgFrame then self.bgFrame = self:CreateParentFrame("PetBattleTeamBackgroundFrame") end
	
	self:Attach(PetBattleTeamsSettings.attached)
	self:LockLocation(PetBattleTeamsSettings.LockLocation)
	
	if not self.petUnitFrames then  self.petUnitFrames = self:CreatePetUnitFrames() end
	if not self.addTeamButton then self.addTeamButton = self:CreateAddTeamButton("AddPetTeamButton") end
end

-- misc functions
PetBattleTeams.OnEvent = function(self,event,...)
    if event == "ADDON_LOADED" then
        name = ...
        if name == "Blizzard_PetJournal" then 
			
			
		elseif name == "PetBattleTeams" then
			
			local loaded, reason = LoadAddOn("Blizzard_PetJournal")
			self:CreateUI()
			if not PetBattleTeamsSettings  or PetBattleTeamsSettings.version ~= PetBattleTeams.version then
				self:Reset(self.resetTeams)
				self:UpdateAllTeams()
			end
			self:UpdateAllTeams()
			self:UnregisterEvent("ADDON_LOADED")
        end
    end
	if event == "PET_JOURNAL_LIST_UPDATE" then 
		self:UpdateAllTeams()
	end
	if event =="PET_JOURNAL_PET_DELETED" then	
		petID = ...
		--self:RepairTeams(petID)
		self:UpdateAllTeams()
	end
end

StaticPopupDialogs["PBT_TEAM_DELETE"] = {
	text = "Are you sure you want to delete team |cffffd200%s|r?",
	button1 = OKAY,
	button2 = CANCEL,
	OnAccept = function(self)
		PetBattleTeams:RemoveTeam(self.data)
		PetBattleTeams:UpdateAllTeams()
	end,
	timeout = 0,
	exclusive = 1,
	hideOnEscape = 1,
}

function SlashCmdList.PETBATTLETEAMS(msg, editbox)
	id = tonumber(msg)
	if id and id > 0 and id <= #PetBattleTeamsSettings.teams then
		PetBattleTeams:SetTeam(id)
	elseif msg == "reset" then
		PetBattleTeams:Reset(true)
		PetBattleTeams:UpdateAllTeams()
	elseif msg == "attach" then
		PetBattleTeams:Attach(true)
	elseif msg == "detach" then
		PetBattleTeams:Attach(false)
	else
		print("To switch between pet teams use /pbt # or /petbattleteams #, where # is a number between 1 and ".. #PetBattleTeamsSettings.teams)
		print("To reset your PBT setting use /pbt reset")
		print("To attach or detach the frame from the pet journal ui")
		print("Use /pbt attach and /pbt detach")
	end
end

function PrintObject(object,prepend)
	if not prepend then prepend = "   " end
	
	if object and type(object) == "table" then
		for k,v in pairs(object) do 
			if type(v) == "table" then 
				print(prepend..k) 
				PrintObject(v,prepend.."  ")
			else
				print (prepend..k.." : "..tostring(v))
			end
			
		end
	end
end

--constants
SLASH_PETBATTLETEAMS1, SLASH_PETBATTLETEAMS2 = '/pbt', '/petbattleteams' 

PetBattleTeams.version = 14
PetBattleTeams.resetTeams = false;
PetBattleTeams.maxTeams = 13
PetBattleTeams.step = 5

--Begin addon
PetBattleTeams:RegisterEvent("PET_JOURNAL_PET_DELETED")
PetBattleTeams:RegisterEvent("PET_JOURNAL_LIST_UPDATE")
PetBattleTeams:RegisterEvent("ADDON_LOADED")
PetBattleTeams:SetScript("OnEvent" , PetBattleTeams.OnEvent)

hooksecurefunc(C_PetJournal, "SetPetLoadOutInfo", PetBattleTeams.UpdateCurrentTeam)
hooksecurefunc(C_PetJournal, "SetAbility", PetBattleTeams.UpdateCurrentTeam)